# -*- coding: utf-8 -*-
import numpy as np
# functions for working with dates
import datetime
# needed for reading data
from netCDF4 import Dataset
# used for creating plots
import matplotlib.pyplot as plt
import seaborn as sns
# for smoothing the visualization of the TUTT
from statsmodels.nonparametric.smoothers_lowess import lowess
# mapping tools are in cartopy
import cartopy.crs as ccrs
import cartopy.feature as cfeat
from cartopy.mpl.ticker import LongitudeFormatter, LatitudeFormatter

# plot parameters
sns.set_style('white')
xfr = LongitudeFormatter(zero_direction_label=True)
yfr = LatitudeFormatter()

# directories
ddir = '../data/'
fdir = '../figs/'

# function to return month given modified Julian day
def mjd2month(mjd,y0=1858,m0=11,d0=17):
    """
    A function for finding the month of a date expressed in
    days relative to some baseline date (can be modified to
    get other aspects of the date as well)

    Variables:
    mjd    :: relative date (scalar)
     (note: arrays will not work for mjd under this approach)

    Parameters:
    y0     :: year of baseline date
    m0     :: month of baseline date
    d0     :: day of baseline date

    Note:
    The default date used in this function corresponds to
    the modified Julian date used by the IBTrACS data set.
    """
    date0 = datetime.datetime(year=y0, month=m0, day=d0)
    date1 = date0 + datetime.timedelta(days=mjd)
    return date1.month

# read in IBTraCS data (center 10 is JTWC - WP)
ncdf = Dataset(ddir+'Basin.WP.ibtracs_all.v03r08.nc')
year = ncdf.variables['season'][:]           # year
genb = ncdf.variables['genesis_basin'][:]    # 2 = Western North Pacific
tc_v = ncdf.variables['source_wind'][:,:,10] # wind speed, in knots
tc_y = ncdf.variables['source_lat'][:,:,10]  # latitude of storm center
tc_x = ncdf.variables['source_lon'][:,:,10]  # longitude of storm center
tc_t = ncdf.variables['source_time'][:,:]    # modified Julian day (6-hourly)
ttyp = ncdf.variables['track_type'][:]       # 0,1 include cyclogenesis
ncdf.close()

# read in JRA-55 wind data
ncdf = Dataset(ddir+'fig1_uwd.nc4')
jlon = ncdf.variables['lon'][:]
jlat = ncdf.variables['lat'][:]
u850 = ncdf.variables['ugrd'][:,0,:,:]
u200 = ncdf.variables['ugrd'][:,1,:,:]
ncdf.close()
ncdf = Dataset(ddir+'fig1_vwd.nc4')
v200 = ncdf.variables['vgrd'][:,0,:,:]
ncdf.close()
# calculate vertical wind shear
ushr = u850-u200

# loop through storms and find genesis locations
cgx = []; cgy = []; cgm = []
# start with all tracks that include cyclogenesis in the WP
sdx = np.where(((ttyp == 0) | (ttyp == 1)) & (genb == 2))[0]
# note that loop is over indices, not over a range!
for ss in sdx:
    # check to see if the wind ever exceeds 25 knots
    if np.any(tc_v[ss,:] >= 25):
        cg = np.where(tc_v[ss,:] >= 25)[0][0]
        # append lat/lon/month of cyclogenesis
        cgy.append(tc_y[ss,cg])
        cgx.append(tc_x[ss,cg])
        cgm.append(mjd2month(tc_t[ss,cg]))
# convert to arrays
cgy = np.array(cgy)
cgx = np.array(cgx)
cgm = np.array(cgm)

# longitude in IBTraCS is (-180,180); convert to (0,360)
cgx[cgx < 0] += 360

# for identifying TUTT location
xdx = np.where((jlon >= 120) & (jlon <= 240))[0]

#===============================================================================
# MAPS (FIG. 1)
#
axp = ccrs.PlateCarree(central_longitude=180)
prj = ccrs.PlateCarree(central_longitude=0)
fig = plt.figure(figsize=(10, 6))
# f.subplots_adjust(hspace=0.25)
# contour levels for climatological vertical wind shear
clv = np.linspace(-30,30,11)

# parameters for all plots
spd = np.sqrt(u200*u200 + v200*v200)
lws = 2*spd/spd.max()
dns = 4
sze = 10

# June
axa = fig.add_subplot(321, projection=axp)
# zonal wind shear
cs0 = axa.contourf(jlon, jlat, ushr[0,:,:], clv, cmap=plt.cm.PuOr, extend='both', transform=prj)
# winds
axa.streamplot(jlon, jlat, u200[0,:,:], v200[0,:,:], linewidth=lws[0,:,:], color='k', density=dns, transform=prj)
# TC genesis locations
idx = np.where(cgm==6)
axa.scatter(cgx[idx], cgy[idx], marker='o', c='#377eb8', s=sze, zorder=10, transform=prj)
# TUTT
ttt = np.empty(20)
for yy in range(45,65):
    dux = np.gradient(np.squeeze(u200[0,yy,xdx]))
    ttt[yy-45] = jlon[xdx[dux.argmax()]]
tut = lowess(ttt, jlat[45:65], frac=0.75, return_sorted=False)
axa.plot(tut, jlat[45:65], color='#999999', linewidth=3, transform=prj)
# plot
axa.set_title(r'June')
axa.set_extent([100, 240, 0, 40], prj)
axa.set_xticks(range(100, 241, 20), crs=prj)
axa.set_yticks(range(0, 41, 10), crs=prj)
axa.xaxis.set_major_formatter(xfr)
axa.yaxis.set_major_formatter(yfr)
axa.add_feature(cfeat.COASTLINE, edgecolor='#333333')

# July
axb = fig.add_subplot(322, projection=axp)
# zonal wind shear
cs0 = axb.contourf(jlon, jlat, ushr[1,:,:], clv, cmap=plt.cm.PuOr, extend='both', transform=prj)
# UT winds
axb.streamplot(jlon, jlat, u200[1,:,:], v200[1,:,:], linewidth=lws[1,:,:], color='k', density=dns, transform=prj)
# TC genesis locations
idx = np.where(cgm==7)
axb.scatter(cgx[idx], cgy[idx], marker='o', c='#377eb8', s=sze, zorder=10, transform=prj)
# TUTT
ttt = np.empty(20)
for yy in range(45,65):
    dux = np.gradient(np.squeeze(u200[1,yy,xdx]))
    ttt[yy-45] = jlon[xdx[dux.argmax()]]
tut = lowess(ttt, jlat[45:65], frac=0.75, return_sorted=False)
axb.plot(tut, jlat[45:65], color='#999999', linewidth=3, transform=prj)
# plot
axb.set_title(r'July')
axb.set_extent([100, 240, 0, 40], prj)
axb.set_xticks(range(100, 241, 20), crs=prj)
axb.set_yticks(range(0, 41, 10), crs=prj)
axb.xaxis.set_major_formatter(xfr)
axb.yaxis.set_major_formatter(yfr)
axb.add_feature(cfeat.COASTLINE, edgecolor='#333333')

# August
axc = fig.add_subplot(323, projection=axp)
# zonal wind shear
cs0 = axc.contourf(jlon, jlat, ushr[2,:,:], clv, cmap=plt.cm.PuOr, extend='both', transform=prj)
# winds
axc.streamplot(jlon, jlat, u200[2,:,:], v200[2,:,:], linewidth=lws[2,:,:], color='k', density=dns, transform=prj)
# TC genesis locations
idx = np.where(cgm==8)
axc.scatter(cgx[idx], cgy[idx], marker='o', c='#377eb8', s=sze, zorder=10, transform=prj)
# TUTT
ttt = np.empty(20)
for yy in range(45,65):
    dux = np.gradient(np.squeeze(u200[2,yy,xdx]))
    ttt[yy-45] = jlon[xdx[dux.argmax()]]
tut = lowess(ttt, jlat[45:65], frac=0.75, return_sorted=False)
axc.plot(tut, jlat[45:65], color='#999999', linewidth=3, transform=prj)
# plot
axc.set_title(r'August')
axc.set_extent([100, 240, 0, 40], prj)
axc.set_xticks(range(100, 241, 20), crs=prj)
axc.set_yticks(range(0, 41, 10), crs=prj)
axc.xaxis.set_major_formatter(xfr)
axc.yaxis.set_major_formatter(yfr)
axc.add_feature(cfeat.COASTLINE, edgecolor='#333333')

# September
axd = fig.add_subplot(324, projection=axp)
# zonal wind shear
cs0 = axd.contourf(jlon, jlat, ushr[3,:,:], clv, cmap=plt.cm.PuOr, extend='both', transform=prj)
# winds
axd.streamplot(jlon, jlat, u200[3,:,:], v200[3,:,:], linewidth=lws[3,:,:], color='k', density=dns, transform=prj)
# TC genesis locations
idx = np.where(cgm==9)
axd.scatter(cgx[idx], cgy[idx], marker='o', c='#377eb8', s=sze, zorder=10, transform=prj)
# TUTT
ttt = np.empty(20)
for yy in range(45,65):
    dux = np.gradient(np.squeeze(u200[3,yy,xdx]))
    ttt[yy-45] = jlon[xdx[dux.argmax()]]
tut = lowess(ttt, jlat[45:65], frac=0.75, return_sorted=False)
axd.plot(tut, jlat[45:65], color='#999999', linewidth=3, transform=prj)
# plot
axd.set_title(r'September')
axd.set_extent([100, 240, 0, 40], prj)
axd.set_xticks(range(100, 241, 20), crs=prj)
axd.set_yticks(range(0, 41, 10), crs=prj)
axd.xaxis.set_major_formatter(xfr)
axd.yaxis.set_major_formatter(yfr)
axd.add_feature(cfeat.COASTLINE, edgecolor='#333333')

# October
axe = fig.add_subplot(325, projection=axp)
# zonal wind shear
cs0 = axe.contourf(jlon, jlat, ushr[4,:,:], clv, cmap=plt.cm.PuOr, extend='both', transform=prj)
# winds
axe.streamplot(jlon, jlat, u200[4,:,:], v200[4,:,:], linewidth=lws[4,:,:], color='k', density=dns, transform=prj)
# TC genesis locations
idx = np.where(cgm==7)
axe.scatter(cgx[idx], cgy[idx], marker='o', c='#377eb8', s=sze, zorder=10, transform=prj)
# TUTT
ttt = np.empty(20)
for yy in range(45,65):
    dux = np.gradient(np.squeeze(u200[4,yy,xdx]))
    ttt[yy-45] = jlon[xdx[dux.argmax()]]
tut = lowess(ttt, jlat[45:65], frac=0.75, return_sorted=False)
axe.plot(tut, jlat[45:65], color='#999999', linewidth=3, transform=prj)
# plot
axe.set_title(r'October')
axe.set_extent([100, 240, 0, 40], prj)
axe.set_xticks(range(100, 241, 20), crs=prj)
axe.set_yticks(range(0, 41, 10), crs=prj)
axe.xaxis.set_major_formatter(xfr)
axe.yaxis.set_major_formatter(yfr)
axe.add_feature(cfeat.COASTLINE, edgecolor='#333333')

# November
axf = fig.add_subplot(326, projection=axp)
# zonal wind shear
cs0 = axf.contourf(jlon, jlat, ushr[5,:,:], clv, cmap=plt.cm.PuOr, extend='both', transform=prj)
# winds
axf.streamplot(jlon, jlat, u200[5,:,:], v200[5,:,:], linewidth=lws[5,:,:], color='k', density=dns, transform=prj)
# TC genesis locations
idx = np.where(cgm==11)
axf.scatter(cgx[idx], cgy[idx], marker='o', c='#377eb8', s=sze, zorder=10, transform=prj)
# TUTT
ttt = np.empty(20)
for yy in range(45,65):
    dux = np.gradient(np.squeeze(u200[5,yy,xdx]))
    ttt[yy-45] = jlon[xdx[dux.argmax()]]
tut = lowess(ttt, jlat[45:65], frac=0.75, return_sorted=False)
axf.plot(tut, jlat[45:65], color='#999999', linewidth=3, transform=prj)
# plot
axf.set_title(r'November')
axf.set_extent([100, 240, 0, 40], prj)
axf.set_xticks(range(100, 241, 20), crs=prj)
axf.set_yticks(range(0, 41, 10), crs=prj)
axf.xaxis.set_major_formatter(xfr)
axf.yaxis.set_major_formatter(yfr)
axf.add_feature(cfeat.COASTLINE, edgecolor='#333333')
plt.savefig(fdir+'fig1_maps.pdf', bbox_inches='tight')

fig = plt.figure(figsize=(13, 4))
# June
axs = fig.add_subplot(111, projection=axp)
# zonal wind shear
cs0 = axs.contourf(jlon, jlat, ushr[0,:,:], clv, cmap=plt.cm.PuOr, extend='both', transform=prj)
# winds
axs.streamplot(jlon, jlat, u200[0,:,:], v200[0,:,:], linewidth=lws[0,:,:]*2, color='k', density=dns, transform=prj)
# TC genesis locations
idx = np.where(cgm==6)
axs.scatter(cgx[idx], cgy[idx], marker='o', c='#377eb8', s=50, zorder=10, transform=prj)
# plot
axs.set_title(r'June')
axs.set_extent([100, 240, 0, 40], prj)
axs.set_xticks(range(100, 241, 20), crs=prj)
axs.set_yticks(range(0, 41, 10), crs=prj)
axs.xaxis.set_major_formatter(xfr)
axs.yaxis.set_major_formatter(yfr)
axs.add_feature(cfeat.COASTLINE, edgecolor='#333333')
plt.savefig(fdir+'fig1a_junemap.pdf', bbox_inches='tight')