# -*- coding: utf-8 -*-
import numpy as np
# functions for working with dates
import datetime
# use pandas for time series analysis
import pandas as pd
import statsmodels.formula.api as smf
# needed for reading data
from netCDF4 import Dataset
# used for creating plots
import matplotlib.pyplot as plt
import seaborn as sns

# plot parameters
sns.set_style('darkgrid')

# directories
ddir = '../data/'
fdir = '../figs/'

# function to return month given modified Julian day
def mjd2month(mjd,y0=1858,m0=11,d0=17):
    """
    A function for finding the month of a date expressed in
    days relative to some baseline date (can be modified to
    get other aspects of the date as well)

    Variables:
    mjd    :: relative date (scalar)
     (note: arrays will not work for mjd under this approach)

    Parameters:
    y0     :: year of baseline date
    m0     :: month of baseline date
    d0     :: day of baseline date

    Note:
    The default date used in this function corresponds to
    the modified Julian date used by the IBTrACS data set.
    """
    date0 = datetime.datetime(year=y0, month=m0, day=d0)
    date1 = date0 + datetime.timedelta(days=mjd)
    return date1.month

# read in IBTraCS data (center 10 is JTWC - WP)
ncdf = Dataset(ddir+'Basin.WP.ibtracs_all.v03r08.nc')
year = ncdf.variables['season'][:]           # year
genb = ncdf.variables['genesis_basin'][:]    # 2 = Western North Pacific
tc_v = ncdf.variables['source_wind'][:,:,10] # wind speed, in knots
tc_x = ncdf.variables['source_lon'][:,:,10]  # longitude of storm center
tc_t = ncdf.variables['source_time'][:,:]    # modified Julian day (6-hourly)
ttyp = ncdf.variables['track_type'][:]       # 0,1 include cyclogenesis
ncdf.close()

# read in JRA-55 wind data
ncdf = Dataset(ddir+'fig2_uwd_tutt.nc4')
jlon = ncdf.variables['lon'][:]
u200 = np.squeeze(ncdf.variables['ugrd'][:])
ncdf.close()

# read in Nino data
ncdf = Dataset(ddir+'fig2_nino34.nc')
nino = np.squeeze(ncdf.variables['sst'][:-1])
ncdf.close()

# loop through storms and find genesis locations
cgx = []; cgt = []; cgm = []
# start with all tracks that include cyclogenesis in the WP
sdx = np.where(((ttyp == 0) | (ttyp == 1)) & (genb == 2))[0]
# note that loop is over indices, not over a range!
for ss in sdx:
    # check to see if the wind ever exceeds 25 knots
    if np.any(tc_v[ss,:] >= 25):
        cg = np.where(tc_v[ss,:] >= 25)[0][0]
        # append lon/month/year of cyclogenesis
        cgx.append(tc_x[ss,cg])
        cgt.append(year[ss])
        cgm.append(mjd2month(tc_t[ss,cg]))
cgt = np.array(cgt)
cgx = np.array(cgx)
cgm = np.array(cgm)
# longitude in IBTraCS is (-180,180); convert to (0,360)
cgx[cgx < 0] += 360

# get mean longitude of cyclogenesis for each year
yrs = np.arange(1958,2015)
mln = np.empty(yrs.shape)
for ii in range(len(yrs)):
    idx = np.where((cgt == yrs[ii]) & (cgm >= 6) & (cgm <= 11))[0]
    if idx.any():
        mln[ii] = cgx[idx].mean()

# get location of TUTT (defined as transition from easterlies to westerlies)
xdx  = np.where((jlon >= 120) & (jlon <= 240))[0]
tutt = np.empty(yrs.shape)
for ii in range(len(yrs)):
    u = u200[ii,xdx]
    x = jlon[xdx]
    tutt[ii] = x[np.where(u > 0)[0][0]]

# make a pandas dataframe
df = pd.DataFrame({'jtwc':mln, 'tutt':tutt, 'nino':nino, 'year':yrs})
# regress JTWC mean longitude of cyclogenesis against Nino 3.4
rgr = smf.ols('jtwc ~ nino', data=df).fit()
# remove ENSO component of variability
df['estm'] = df['jtwc'].mean() + (df['jtwc']-rgr.predict(df['nino']))

# check trend in TUTT and TC genesis longitude
trd = smf.ols('tutt ~ year', data=df.ix[21:]).fit()
print trd.summary()
trd = smf.ols('jtwc ~ year', data=df.ix[21:]).fit()
print trd.summary()
trd = smf.ols('estm ~ year', data=df.ix[21:]).fit()
print trd.summary()

#===============================================================================
# PLOT TIME SERIES OF MEAN LONGITUDE OF CG AND TUTT (FIG. 2)
#
fig = plt.figure(figsize=(12,6))
axa = fig.add_subplot(211)
axa.plot(df['year'], df['jtwc'], '-', color='#377eb8')
sns.regplot(df['year'], df['jtwc'], color='#377eb8', ax=axa, truncate=True)
sns.regplot(df.ix[21:]['year'], df.ix[21:]['jtwc'], color='#377eb8',
            line_kws={'linestyle':'--'}, truncate=True, ci=None, ax=axa)
axa.set_xlabel('')
axa.set_xlim(1955,2015)
axa.set_xticks(range(1960, 2011, 10))
axa.set_ylabel(u'TC mean genesis longitude [°E]', color='#377eb8')
axa.set_ylim(125,150)
axa.set_yticks(range(125,151,5))
axa.set_yticklabels(range(125,151,5), color='#377eb8')
axb = axa.twinx()
axb.plot(df['year'], df['tutt'], '-', color='#e41a1c')
sns.regplot(df['year'], df['tutt'], color='#e41a1c', ax=axb, truncate=True)
sns.regplot(df.ix[21:]['year'], df.ix[21:]['tutt'], color='#e41a1c',
            line_kws={'linestyle':'--'}, truncate=True, ci=None, ax=axb)
axb.set_xlim(1955,2015)
axb.set_xticks(range(1960, 2011, 10))
axb.set_ylabel(u'TUTT longitude [°E]', color='#e41a1c')
axb.set_ylim(140,190)
axb.set_yticks(range(140,191,10))
axb.set_yticklabels(range(140,191,10), color='#e41a1c')
axb.grid(False)
axa.text(1962, 127, r'$R$ = {0:.2f}'.format(df.corr()['jtwc']['tutt']))
axc = fig.add_subplot(212)
axc.plot(df['year'], df['jtwc'], '-', color='#377eb8', label='JTWC (original)')
sns.regplot(df['year'], df['jtwc'], color='#377eb8', ci=None, ax=axc, truncate=True)
sns.regplot(df.ix[21:]['year'], df.ix[21:]['jtwc'], color='#377eb8',
            line_kws={'linestyle':'--'}, truncate=True, ci=None, ax=axc)
axc.plot(df['year'], df['estm'], '-', color='#4daf4a', label='JTWC (ENSO influence removed)')
sns.regplot(df['year'], df['estm'], color='#4daf4a', ci=None, ax=axc, truncate=True)
sns.regplot(df.ix[21:]['year'], df.ix[21:]['estm'], color='#4daf4a',
            line_kws={'linestyle':'--'}, truncate=True, ci=None, ax=axc)
axc.set_xlabel('Year')
axc.set_xlim(1955,2015)
axc.set_xticks(range(1960, 2011, 10))
axc.set_ylabel(u'TC mean genesis longitude [°E]')
axc.set_ylim(120,170)
axc.set_yticks(range(120,171,10))
axc.set_yticklabels(range(120,171,10))
axc.legend(loc=2)
plt.savefig(fdir+'fig2_tutt.pdf', bbox_inches='tight')

fig = plt.figure(figsize=(13, 4))
axa = fig.add_subplot(111)
axa.plot(df['year'], df['jtwc'], '-', color='#377eb8')
sns.regplot(df['year'], df['jtwc'], color='#377eb8', ax=axa, truncate=True)
sns.regplot(df.ix[21:]['year'], df.ix[21:]['jtwc'], color='#377eb8',
            line_kws={'linestyle':'--'}, truncate=True, ci=None, ax=axa)
axa.set_xlabel('')
axa.set_xlim(1955,2015)
axa.set_xticks(range(1960, 2011, 10))
axa.set_ylabel(u'TC mean genesis longitude [°E]', color='#377eb8')
axa.set_ylim(125,150)
axa.set_yticks(range(125,151,5))
axa.set_yticklabels(range(125,151,5), color='#377eb8')
axb = axa.twinx()
axb.plot(df['year'], df['tutt'], '-', color='#e41a1c')
sns.regplot(df['year'], df['tutt'], color='#e41a1c', ax=axb, truncate=True)
sns.regplot(df.ix[21:]['year'], df.ix[21:]['tutt'], color='#e41a1c',
            line_kws={'linestyle':'--'}, truncate=True, ci=None, ax=axb)
axb.set_xlim(1955,2015)
axb.set_xticks(range(1960, 2011, 10))
axb.set_ylabel(u'TUTT longitude [°E]', color='#e41a1c')
axb.set_ylim(140,190)
axb.set_yticks(range(140,191,10))
axb.set_yticklabels(range(140,191,10), color='#e41a1c')
axb.grid(False)
axa.text(1962, 127, r'$R$ = {0:.2f}'.format(df.corr()['jtwc']['tutt']))
plt.savefig(fdir+'fig2a_tutt.pdf', bbox_inches='tight')

fig = plt.figure(figsize=(13, 4))
axc = fig.add_subplot(111)
axc.plot(df['year'], df['jtwc'], '-', color='#377eb8', label='JTWC (original)')
sns.regplot(df['year'], df['jtwc'], color='#377eb8', ci=None, ax=axc, truncate=True)
sns.regplot(df.ix[21:]['year'], df.ix[21:]['jtwc'], color='#377eb8',
            line_kws={'linestyle':'--'}, truncate=True, ci=None, ax=axc)
axc.plot(df['year'], df['estm'], '-', color='#4daf4a', label='JTWC (ENSO influence removed)')
sns.regplot(df['year'], df['estm'], color='#4daf4a', ci=None, ax=axc, truncate=True)
sns.regplot(df.ix[21:]['year'], df.ix[21:]['estm'], color='#4daf4a',
            line_kws={'linestyle':'--'}, truncate=True, ci=None, ax=axc)
axc.set_xlabel('Year')
axc.set_xlim(1955,2015)
axc.set_xticks(range(1960, 2011, 10))
axc.set_ylabel(u'TC mean genesis longitude [°E]')
axc.set_ylim(120,160)
axc.set_yticks(range(120,161,10))
axc.set_yticklabels(range(120,161,10))
axc.legend(loc=2)
plt.savefig(fdir+'fig2b_enso.pdf', bbox_inches='tight')