# -*- coding: utf-8 -*-
import numpy as np
# functions for working with dates
import datetime
# use pandas for time series analysis
import pandas as pd
import statsmodels.api as sm
# needed for reading data
from netCDF4 import Dataset
# used for creating plots
import matplotlib.pyplot as plt
import seaborn as sns

# plot parameters
sns.set_style('darkgrid')

# directories
ddir = '../data/'
fdir = '../figs/'

# function to return month given modified Julian day
def mjd2month(mjd,y0=1858,m0=11,d0=17):
    """
    A function for finding the month of a date expressed in
    days relative to some baseline date (can be modified to
    get other aspects of the date as well)

    Variables:
    mjd    :: relative date (scalar)
     (note: arrays will not work for mjd under this approach)

    Parameters:
    y0     :: year of baseline date
    m0     :: month of baseline date
    d0     :: day of baseline date

    Note:
    The default date used in this function corresponds to
    the modified Julian date used by the IBTrACS data set.
    """
    date0 = datetime.datetime(year=y0, month=m0, day=d0)
    date1 = date0 + datetime.timedelta(days=mjd)
    return date1.month

# read in IBTraCS data (center 10 is JTWC - WP)
ncdf = Dataset(ddir+'Basin.WP.ibtracs_all.v03r08.nc')
year = ncdf.variables['season'][:]           # year
genb = ncdf.variables['genesis_basin'][:]    # 2 = Western North Pacific
tc_v = ncdf.variables['source_wind'][:,:,10] # wind speed, in knots
tc_x = ncdf.variables['source_lon'][:,:,10]  # longitude of storm center
tc_t = ncdf.variables['source_time'][:,:]    # modified Julian day (6-hourly)
ttyp = ncdf.variables['track_type'][:]       # 0,1 include cyclogenesis
ncdf.close()

# loop through storms and find genesis locations
cgx = []; cgt = []; cgm = []
# start with all tracks that include cyclogenesis in the WP
sdx = np.where(((ttyp == 0) | (ttyp == 1)) & (genb == 2))[0]
# note that loop is over indices, not over a range!
for ss in sdx:
    # check to see if the wind ever exceeds 25 knots
    if np.any(tc_v[ss,:] >= 25):
        cg = np.where(tc_v[ss,:] >= 25)[0][0]
        # append lon/month/year of cyclogenesis
        cgx.append(tc_x[ss,cg])
        cgt.append(year[ss])
        cgm.append(mjd2month(tc_t[ss,cg]))
cgt = np.array(cgt)
cgx = np.array(cgx)
cgm = np.array(cgm)
# longitude in IBTraCS is (-180,180); convert to (0,360)
cgx[cgx < 0] += 360

# get number of TCs forming in the eastern/western WNP for each year
yrs = np.arange(1958,2015)
w_n = np.empty(yrs.shape)
e_n = np.empty(yrs.shape)
for ii in range(len(yrs)):
    idx = np.where((cgt == yrs[ii]) & (cgm >= 6) & (cgm <= 11))[0]
    if idx.any():
        w_n[ii] = (cgx[idx] < 145).sum()
        e_n[ii] = (cgx[idx] >= 145).sum()

# data frame for panel (a)
dfa = pd.DataFrame({'wwnp':w_n, 'ewnp':e_n}, index=yrs)

# read in JRA-55 wind data
ncdf = Dataset(ddir+'fig3_uwd.nc4')
u850 = np.squeeze(ncdf.variables['ugrd'][:,0,:,:])
u200 = np.squeeze(ncdf.variables['ugrd'][:,1,:,:])
ncdf.close()
# calculate vertical wind shear (omit data for 2015)
ushr = (u200-u850)[:-1]

# read in JRA-55 temperature data
ncdf = Dataset(ddir+'fig3_tmp_tropics.nc4')
ttmp = np.squeeze(ncdf.variables['tmp'][:])
ncdf.close()
ncdf = Dataset(ddir+'fig3_tmp_subtrop.nc4')
stmp = np.squeeze(ncdf.variables['tmp'][:])
ncdf.close()
# calculate meridional temperature gradient (omit data for 2015)
tgrd = (ttmp-stmp)[:-1]

# data frame for panel (b)
dfb = pd.DataFrame({'ushr':ushr, 'tgrd':tgrd}, index=yrs)

#===============================================================================
# PLOT TIME SERIES OF MEAN LONGITUDE OF CG AND TUTT (FIG. 2)
#
# plot keywords
skw = {'marker':'o', 'edgecolors':'none', 's':20, 'label':''}
# figure
fig = plt.figure(figsize=(12,6))
axa = fig.add_subplot(211)
axa.plot(dfa.index, dfa['wwnp'], '-', color='#7570b3', label='Western WNP')
axa.scatter(dfa.index, dfa['wwnp'], c='#7570b3', **skw)
sns.regplot(dfa.ix[1979:].index, dfa.ix[1979:]['wwnp'], color='#7570b3',
            line_kws={'linestyle':'--'}, truncate=True, ax=axa)
axa.plot(dfa.index, dfa['ewnp'], '-', color='#d95f02', label='Eastern WNP')
axa.scatter(dfa.index, dfa['ewnp'], c='#d95f02', **skw)
sns.regplot(dfa.ix[1979:].index, dfa.ix[1979:]['ewnp'], color='#d95f02',
            line_kws={'linestyle':'--'}, truncate=True, ax=axa)
axa.set_xlabel('')
axa.set_xlim(1955,2015)
axa.set_xticks(range(1960, 2011, 10))
axa.set_ylabel('TC count')
axa.set_ylim(0,30)
axa.set_yticks(range(0,31,5))
axa.text(1962, 1.25, r'$R$ = $-${0:.2f}'.format(-dfa.corr()['wwnp']['ewnp']))
axa.legend(loc=1)
axb = fig.add_subplot(212)
axb.plot(dfb.index, dfb['tgrd'], '-', color='#e7298a', label='Temperature gradient')
sns.regplot(dfb.index, dfb['tgrd'], color='#e7298a', ci=None, ax=axb, truncate=True)
sns.regplot(dfb.ix[1979:].index, dfb.ix[1979:]['tgrd'], color='#e7298a',
            line_kws={'linestyle':'--'}, truncate=True, ci=None, ax=axb)
axb.set_xlabel('Year')
axb.set_xlim(1955,2015)
axb.set_xticks(range(1960, 2011, 10))
axb.set_ylabel(u'Temperature gradient [K]', color='#e7298a')
axb.set_ylim(0,2.1)
axb.set_yticks(np.arange(0,2.11,0.3))
axb.set_yticklabels(np.arange(0,2.11,0.3), color='#e7298a')
axc = axb.twinx()
axc.plot(dfb.index, dfb['ushr'], '-', color='#1b9e77', label='Zonal wind shear')
sns.regplot(dfb.index, dfb['ushr'], color='#1b9e77', ci=None, ax=axc, truncate=True)
sns.regplot(dfb.ix[1979:].index, dfb.ix[1979:]['ushr'], color='#1b9e77',
            line_kws={'linestyle':'--'}, truncate=True, ci=None, ax=axc)
axc.set_ylabel(r'Zonal wind shear [m s$^{-\mathregular{1}}$]', color='#1b9e77')
axc.set_ylim(0,14)
axc.set_yticks(np.arange(0,15,2))
axc.set_yticklabels(range(0,15,2), color='#1b9e77')
axc.grid(False)
axb.text(2005, 0.4, r'$R$ = {0:.2f}'.format(dfb.corr()['ushr']['tgrd']), ha='center')
plt.savefig(fdir+'fig3_thermwind.pdf', bbox_inches='tight')

fig = plt.figure(figsize=(13, 4))
axa = fig.add_subplot(111)
axa.plot(dfa.index, dfa['wwnp'], '-', color='#7570b3', label='Western WNP')
axa.scatter(dfa.index, dfa['wwnp'], c='#7570b3', **skw)
sns.regplot(dfa.ix[1979:].index, dfa.ix[1979:]['wwnp'], color='#7570b3',
            line_kws={'linestyle':'--'}, truncate=True, ax=axa)
axa.plot(dfa.index, dfa['ewnp'], '-', color='#d95f02', label='Eastern WNP')
axa.scatter(dfa.index, dfa['ewnp'], c='#d95f02', **skw)
sns.regplot(dfa.ix[1979:].index, dfa.ix[1979:]['ewnp'], color='#d95f02',
            line_kws={'linestyle':'--'}, truncate=True, ax=axa)
axa.set_xlabel('')
axa.set_xlim(1955,2015)
axa.set_xticks(range(1960, 2011, 10))
axa.set_ylabel('TC count')
axa.set_ylim(0,30)
axa.set_yticks(range(0,31,5))
axa.text(1962, 1.25, r'$R$ = $-${0:.2f}'.format(-dfa.corr()['wwnp']['ewnp']))
axa.legend(loc=1)
plt.savefig(fdir+'fig3a_wnpcounts.pdf', bbox_inches='tight')

fig = plt.figure(figsize=(13, 4))
axb = fig.add_subplot(111)
axb.plot(dfb.index, dfb['tgrd'], '-', color='#e7298a', label='Temperature gradient')
sns.regplot(dfb.index, dfb['tgrd'], color='#e7298a', ci=None, ax=axb, truncate=True)
sns.regplot(dfb.ix[1979:].index, dfb.ix[1979:]['tgrd'], color='#e7298a',
            line_kws={'linestyle':'--'}, truncate=True, ci=None, ax=axb)
axb.set_xlabel('Year')
axb.set_xlim(1955,2015)
axb.set_xticks(range(1960, 2011, 10))
axb.set_ylabel(u'Temperature gradient [K]', color='#e7298a')
axb.set_ylim(0,2.1)
axb.set_yticks(np.arange(0,2.11,0.3))
axb.set_yticklabels(np.arange(0,2.11,0.3), color='#e7298a')
axc = axb.twinx()
axc.plot(dfb.index, dfb['ushr'], '-', color='#1b9e77', label='Zonal wind shear')
sns.regplot(dfb.index, dfb['ushr'], color='#1b9e77', ci=None, ax=axc, truncate=True)
sns.regplot(dfb.ix[1979:].index, dfb.ix[1979:]['ushr'], color='#1b9e77',
            line_kws={'linestyle':'--'}, truncate=True, ci=None, ax=axc)
axc.set_ylabel(r'Zonal wind shear [m s$^{-\mathregular{1}}$]', color='#1b9e77')
axc.set_ylim(0,14)
axc.set_yticks(np.arange(0,15,2))
axc.set_yticklabels(range(0,15,2), color='#1b9e77')
axc.grid(False)
axb.text(2005, 0.4, r'$R$ = {0:.2f}'.format(dfb.corr()['ushr']['tgrd']), ha='center')
plt.savefig(fdir+'fig3b_thermwind.pdf', bbox_inches='tight')