# -*- coding: utf-8 -*-
import numpy as np
# use pandas for time series analysis
import pandas as pd
import statsmodels.api as sm
# needed for reading data
from netCDF4 import Dataset
# used for creating plots
import matplotlib.pyplot as plt
import seaborn as sns

# plot parameters
sns.set_style('white')

# directories
ddir = '../data/'
fdir = '../figs/'

# function to calculate trend and significance from a pandas time series
def pdtrend(x,ci=0.95):
    """
    A basic function for calculating trends given a pandas time
    series.

    Variables:
    x      :: the series

    Parameters:
    ci     :: confidence interval for significance testing

    Note:
    The example paper we are working from uses a non-parametric
    Mann-Kendall test, and accounts for auto-correlation in the
    time series in significance testing. This function uses the
    Student's t test, which is (a) parametric and (b) assumes
    that the underlying data are normally distributed. This
    approach is acceptable for exploratory analysis, but would
    be unsuitable for publication-quality work. For more details,
    see Chapter 17 of Statistical Analysis in Climate Research
    by Hans von Storch and Francis W. Zwiers.
    """
    # statsmodels regression require us to add a constant
    xvl = sm.add_constant(np.array(x.index))
    # ordinary least squares linear regression
    rgr = sm.OLS(x.values,xvl).fit()
    # trend slope (per year in this case)
    trnd = rgr.params[1]
    # simple representation of statistical significance (returns True/False)
    tsig = (rgr.pvalues[1] < (1-ci))
    return (trnd, tsig)

# read in JRA-55 wind data
ncdf = Dataset(ddir+'fig4_uwd_pacmm.nc4')
jlev = ncdf.variables['lev'][:27]*0.01
jlon = ncdf.variables['lon'][:]
mm_u = np.squeeze(ncdf.variables['ugrd'][21:,:27,:,:])
ncdf.close()

# read in JRA-55 temperature data
ncdf = Dataset(ddir+'fig4_tmp_paczm.nc4')
jlev = ncdf.variables['lev'][:27]*0.01
jlat = ncdf.variables['lat'][:]
zm_t = np.squeeze(ncdf.variables['tmp'][21:,:27,:,:])
ncdf.close()

tru = np.ma.masked_all(mm_u.shape[1:])
trt = np.ma.masked_all(zm_t.shape[1:])
for zz in range(tru.shape[0]):
    for xx in range(tru.shape[1]):
        ser = pd.Series(mm_u[:,zz,xx])
        m,s = pdtrend(ser)
        tru.data[zz,xx] = m*10 # convert trend to per decade
        tru.mask[zz,xx] = ~s
    for yy in range(trt.shape[1]):
        ser = pd.Series(zm_t[:,zz,yy])
        m,s = pdtrend(ser)
        trt.data[zz,yy] = m*10 # convert trend to per decade
        trt.mask[zz,yy] = ~s

# longitude-pressure cross section of zonal wind and trends
lonlbl = [u'100°E',u'120°E',u'140°E',u'160°',u'180°',u'160°W']
fig = plt.figure(figsize=(10,7),dpi=72)
ax0 = fig.add_subplot(111)
cs1 = ax0.contour(jlon, jlev, mm_u.mean(axis=0), np.arange(-50,50,2), colors='k')
cs2 = ax0.contourf(jlon, jlev, tru.data, np.linspace(-1,1,11),
                   cmap=plt.cm.RdBu_r, extend='both')
ax0.contourf(jlon, jlev, tru.mask.astype('int'), [-0.5,0.5], hatches=['xxxx','none'],
             colors='none', edgecolor='#666666', zorder=10)
ax0.set_xticks(np.arange(120,201,20))
ax0.set_xticklabels(lonlbl)
ax0.set_xlim(120,200)
ax0.set_ylim(1000,100)
ax0.set_yscale('log')
ax0.set_yticks([1000,850,700,500,400,300,250,200,150,100])
ax0.set_yticklabels([1000,850,700,500,400,300,250,200,150,100])
ax0.set_ylabel(r'Pressure [hPa]')
cb = plt.colorbar(cs2,orientation='vertical',extend='both',aspect=50)
cb.set_ticks([-1,-0.5,0,0.5,1])
cb.set_label('Trend in zonal wind [m s$^{-\mathregular{1}}$ dec$^{-\mathregular{1}}$]')
plt.savefig(fdir+'fig4a_zonalwindtrend_1979.pdf', bbox_inches='tight')

# latitude-pressure cross section of temperature and trends
latlbl = [u'45°S', u'30°S', u'15°S', u'0°', u'15°N', u'30°N', u'45°N']
fig = plt.figure(figsize=(10,7),dpi=72)
ax0 = fig.add_subplot(111)
cs1 = ax0.contour(jlat, jlev, zm_t.mean(axis=0), np.arange(200,301,10), colors='k')
cs2 = ax0.contourf(jlat, jlev, trt.data, np.linspace(-0.5,0.5,11),
                   cmap=plt.cm.RdBu_r, extend='both')
ax0.contourf(jlat, jlev, trt.mask.astype('int'), [-0.5,0.5], hatches=['xxxx','none'],
             colors='none', edgecolor='#666666', zorder=10)
ax0.set_xticks(np.arange(-45,46,15))
ax0.set_xticklabels(latlbl)
ax0.set_xlim(-45,45)
ax0.set_ylim(1000,100)
ax0.set_yscale('log')
ax0.set_yticks([1000,850,700,500,400,300,250,200,150,100])
ax0.set_yticklabels([1000,850,700,500,400,300,250,200,150,100])
ax0.set_ylabel(r'Pressure [hPa]')
cb = plt.colorbar(cs2,orientation='vertical',extend='both',aspect=50)
cb.set_ticks(np.linspace(-0.5,0.5,11))
cb.set_label('Trend in temperature [K dec$^{-\mathregular{1}}$]')
plt.savefig(fdir+'fig4b_temptrend_1979.pdf', bbox_inches='tight')